#clear the info window
clearinfo

#determine praat version
requiredVersion = 5204
if praatVersion < requiredVersion
    exit Your Praat (Version 'praatVersion') is too old. Use Version 'requiredVersion' or newer.
endif


#wavefile folder path
waveFilePath$ = chooseDirectory$ ("Select folder containing wave files")
if waveFilePath$ == ""
	exit Script exited. You did not select a folder.
else
	waveFilePath$ = waveFilePath$ + "/";
endif

#get additional information from user
beginPause ("TriggerWave")
	comment ("Which channel contains the triggers?")
	choice ("TriggerChannel:", 2)
		option ("left")
		option ("right")
	comment ("Do you have a left trigger boundary?")
	choice ("LeftBoundary", 1)
		option ("yes")
		option ("no")
	comment ("Trigger Threshold. Increase this number if you are getting false triggers.")
	positive ("TriggerThreshold", 40);
	comment ("Maximum sound file length in second")
	comment ("(Decrease this number to avoid out-of-memory error)")
	positive ("MaxLength", 900);
clicked = endPause ("Next", 1);

Create Strings as file list... list 'waveFilePath$'
numOfFiles = Get number of strings
for ifile to numOfFiles
	filename$ = Get string... ifile
	extension$ = right$ (filename$,4)
	
	#loop through all wave files
	if extension$ == ".wav"
		#within this loop, we process each wave file individually	

		#read in the label file
		labelFileNameNoExtension$ = left$(filename$, rindex(filename$, ".")-1)
		labelFileName$ = labelFileNameNoExtension$ + ".txt"
		labelFilePath$ = waveFilePath$ + labelFileName$

		if fileReadable(labelFilePath$)
			Read Strings from raw text file... 'labelFilePath$'
		else
			exit Could not read text file 'labelFilePath$'
		endif

		#read in the wave file
		Open long sound file... 'waveFilePath$''filename$'
		longSound = selected ("LongSound")
		longSoundTotalDuration = Get total duration
		#longSoundEndTime = Get end time
		numberOfSegments = 1;
		if longSoundTotalDuration > maxLength
			numberOfSegments = ceiling (longSoundTotalDuration/maxLength)
		endif

		printline Processing 'waveFilePath$''filename$'

		#extract long sound file into smaller Sound objects for processing
		#this is necessary because Praat has a memory limit that prevents us from loading long Sound objects into memory
		#the workaround here is to load a wave file as a "LongSound" object and then extract smaller "Sound" objects 
		#each small Sound object is processed individually and them removed before the loop is repeated 
		
		j = 1 
		previousTime = 0;
		for k from 1 to numberOfSegments
			shortSoundStartTime = (k-1) * maxLength;
			shortSoundEndTime = (k * maxLength);
			if shortSoundEndTime > longSoundTotalDuration
				shortSoundEndTime = longSoundTotalDuration;
			endif
			select longSound
			Extract part... shortSoundStartTime shortSoundEndTime yes

			stereoSound =  selected ("Sound")
			Extract one channel... 1
			Rename... left
			select stereoSound
			Extract one channel... 2
			Rename... right

			#select the trigger channel
			select Sound 'triggerChannel$'

			#deduce sound channel
			if triggerChannel$ == "left"
				soundChannel$ = "right"
			else
				soundChannel$ = "left"
			endif
			#get total duration
			totalDuration = Get end time

			#create intensity object
			To Intensity... 100 0 yes

			#select intensity object
			select Intensity 'triggerChannel$'

			#prepare output file
			#only prepare output file is this is the first round of the sub loop
			if k == 1
				outFileName$ = labelFileNameNoExtension$ + ".TextGrid"
				outFilePath$ = waveFilePath$ + outFileName$
				deleteFile (outFilePath$)
				filedelete "'outFilePath$'"
				fileappend "'outFilePath$'" "Praat chronological TextGrid text file" 'newline$'
				fileappend "'outFilePath$'" 0 'longSoundTotalDuration'  ! Time domain. 'newline$'
				fileappend "'outFilePath$'" 1  ! Number of tiers. 'newline$'
				fileappend "'outFilePath$'" "IntervalTier" "label" 0 'totalDuration' 'newline$'
			endif

			#loop through all intensity values, check if instensity > triggerThreshold
			#each time intensity > triggerThreshold, consider that a trigger
			#pull time stamp at trigger points, write to praat TextGrid
		
			i = Get start time
			#j = 1
			#previousTime = 0;
			while i <= totalDuration
				amp = Get value at time... i Cubic
				if amp <> undefined and amp >= triggerThreshold
					select Strings 'labelFileNameNoExtension$'
					label$ = Get string... j
				
					#write time stamp to praat text grid
					#take into account left boundary existence
					if j==1
						if leftBoundary$ == "yes"
							fileappend "'outFilePath$'" 1 0 'i' 'newline$'
							fileappend "'outFilePath$'" "" 'newline$'
						else
							fileappend "'outFilePath$'" 1 0 'i' 'newline$'
							fileappend "'outFilePath$'" "'label$'" 'newline$'
						endif
						previousTime = i
					else
						if leftBoundary$ == "yes" 
							label$ = Get string... j-1
						else
							label$ = Get string... j
						endif
						fileappend "'outFilePath$'" 1 'previousTime' 'i' 'newline$'
						fileappend "'outFilePath$'" "'label$'" 'newline$'
						previousTime = i					
					endif		

					select Intensity 'triggerChannel$'
					i += 1
					j += 1
				else
					i += 0.001
				endif
			endwhile
		


			#clean up
			select Intensity 'triggerChannel$'
			Remove
			select Sound right
			Remove
			select Sound left
			Remove
			select stereoSound
			Remove

		endfor


		#write last text grid boundary
		select Strings 'labelFileNameNoExtension$'
		if leftBoundary$ == "yes" 
			label$ = Get string... j-1
		else
			label$ = Get string... j
		endif
		fileappend "'outFilePath$'" 1 'previousTime' 'totalDuration' 'newline$'
		fileappend "'outFilePath$'" "'label$'" 'newline$'
			
		#read in the newly created textgrid
		Read from file... 'waveFilePath$''labelFileNameNoExtension$'.TextGrid

		#reselect the list string so that the wave file loop can continue
		select Strings list
	endif 	

#loop through all wave files ends
endfor

#remove list strings
select Strings list
Remove
select Strings 'labelFileNameNoExtension$'
Remove

#print completed message
print TriggerWave Script Completed


